/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file tlamp.c
 * @brief Simple FT API example for a local lamp
 */

/*
# This test changes lamp brightness
# Lamp must be connected to O7 and GND (O8 also can be used as GND)
import time
import ft
txt2=ft.fttxt2()
l7=txt2.lamp(7)
txt2.update_config()
print("Lamp test")
i=0
while i <= 512:
    l7.pwm_set(i)
    i = i + 20
    time.sleep(1)

l7.pwm_set(0)

*/

#include "ftlock.h"
#include "ft.h"

#include <stdio.h>
#include <unistd.h>

/**
 * @brief Standard main procedure
 * @param [in] argc number of arguments
 * @param [in] argv arguments
 * @return Error code
 */
int main(int argc, char **argv)
{
    if (ftlock_open() < 0)
	return 1;

    if (ftlock_try_lock(argv[0]) < 0)
	return 1;

    void *txt = ftop_alloc_txt("auto", 65000);

    (void)argc; (void)argv;

    if (txt == NULL) {
        fprintf(stderr,"Can't conect %s\n", argv[1]);
        return 1;
    }

    void *l7 = ftop_alloc_lamp(txt, ftdev_o7);

    if (l7 == NULL) {
    	fprintf(stderr,"Can't allocate O7 lamp\n");
    	return 1;
    }

    ftop_update_config(txt);

    ftlock_notify('R');

    fprintf(stderr,"Lamp test\n");

    int i;

    for (i = 0; i <= 512; i += 20) {
    	ftop_set_brightness(l7, i);
    	sleep(1);
    }

    ftop_set_brightness(l7, 0);

    ftlock_notify('F');

    /* These calls are inimportant */
    ftop_free_txt(txt);
    ftlock_unlock();
    ftlock_close();

    return 0;
}

