
/*
 * Copyright (c) 2023 Accso. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file inc/ftctrl.h
 * @brief FT Control utility
 */


// command line inspired by https://github.com/schreinerman/ioCommandLine

#ifndef __FTCMDLINE_H__
#define __FTCMDLINE_H__

/* C binding of definitions if building with C++ compiler */
#ifdef __cplusplus
extern "C"
{
#endif

#include <string.h>
#include <stdio.h>
#include <stdint.h>

#define CMDLINE_MAXBUFFER  512
#define CMDLINE_MAXCMDARGS   5

typedef void (*pfn_ftctrl_cmdline_callback_t)(char* args[], uint8_t argv);

typedef int (*pfn_ftctrl_cmdline_putchar)(char c);

struct stc_ftctrl_cmdline_item;

typedef struct stc_ftctrl_cmdline_item
{
    pfn_ftctrl_cmdline_callback_t pfnCallback;
    char* pcCommand;
    char* pcHelpText;
    struct stc_ftctrl_cmdline_item* nextListItm;
} stc_ftctrl_cmdline_item_t;

void FtCmdLine_Init(void);
void FtCmdLine_AddCommands(stc_ftctrl_cmdline_item_t apfnCommands[], uint8_t u8CommandCount);
void FtCmdLine_Execute(char *pcString);
void FtCmdLine_ExecuteCommand(char* pcCommand,char* pcArgumentString);
void FtCmdLine_ExecuteCommandArgs(char* pcCommand,char* args[],uint8_t argv);
void FtCmdLine_Update(void);
#if CMDLINE_MAXBUFFER > 0
void FtCmdLine_AddChar(char cChar);
void FtCmdLine_AddString(char* pcChar);
#endif

#ifdef __cplusplus
}
#endif

#endif // __FTCMDLINE_H__