/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @defgroup Common_FT_API_Implementation_Includes
 * @{
 */

/**
 * @file ftintshmem.h
 * @brief common shared structures
 */

/**
 * @addtogroup Internal_Shared_Memory_Implementation
 * @{
 */

#ifndef FTINC_FTINTSHMEM_H_
#define FTINC_FTINTSHMEM_H_

#include "ftshmem.h"

/**
 * Remote ftconfig
 */
struct ft_remote_config {
	uint32_t configid;        /*!< current config id */
	uint32_t usage;           /*!< current usage info */
	struct ft_config config;  /*!< I1-I8/M1-M4 config itself */
};

/**
 * Remote values
 */
struct ft_remote_values {
	uint16_t valuesid;        /*!< current values id */
	ftvalues values;          /*!< values itself */
};

/**
 * Remote operation
 */
struct ft_remote_op {
	uint16_t opid;                               /*!< operation id */
	uint16_t usage;                              /*!< usage bitmask */
	uint32_t mcmdid[FT_MAX_MOTORS];              /*!< M1-M4 cmdids */
	uint8_t  ccmdid[FT_MAX_COUNTERS];            /*!< C1-C4 cmdids */
	uint16_t distance[FT_MAX_MOTORS];            /*!< M1/M4 distance */
	uint16_t pwm[FT_MAX_OUTPUTS + FT_MAX_SERVO]; /*!< PWM values for O1-O8 and S1-S3 */
	uint8_t  mmask;                              /*!< group mask for M1-M4 */
};

/**
 * Shared memory structure for remote TXT2 controller
 */
struct ft_shm_txt_remote {
	struct ft_shm_txt common;             /*!< common part for all controllers */
	/* Private part for communications */
	struct ft_remote_config current;      /*!< current remote config */
	struct ft_remote_config last;         /*!< last remote config */
	struct ft_remote_op op;               /*!< current remote operation */
	struct ft_remote_op lop;              /*!< last remote operation */
	uint32_t cmdid;                       /*!< cmdid  for optimization */
	uint32_t valuesid;                    /*!< valuesid for optimization */
};

/**
 * Memory to represent recived master config/op
 */
struct ft_master_remote {
	struct ft_remote_config current;      /*!< current remote config */
	struct ft_remote_config last;         /*!< last remote config */
	struct ft_remote_op op;               /*!< current remote operation */
	struct ft_remote_op lop;              /*!< last remote operation */
};

/** Motor break values */
struct ft_motor_break_info {
	uint32_t ltime;     /*!< last recalculation time */
	uint32_t dtime;     /*!< delta time              */
	uint32_t lcount;    /*!< last counter value      */
	uint8_t  breakmode; /*!< brake motor on          */
	uint8_t  reached;   /*!< reached flag */
};

#endif /* FTINC_FTINTSHMEM_H_ */

/** @} */

/** @} */


