/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file ftservermem.h
 * @brief Main server structure
 */

#ifndef FTINC_FTSERVERMEM_H_
#define FTINC_FTSERVERMEM_H_

#include "ftintshmem.h"

/*!
 * Common shared memory structure available from application
 * First element is always local TXT2
 * It is master for master or slave for slave
 * slaves[0] is first slave for master and master for slave
 */
extern struct ft_shm_server {
	struct ft_shm_header header;
	ftuid uids[FT_MAX_TXT];
	struct ft_shm_txt *txts[FT_MAX_TXT];
	struct ft_shm_txt current;

	/* Local private info */
	uint32_t counters[FT_MAX_COUNTERS];
	uint8_t cstate[FT_MAX_COUNTERS];
	uint32_t fcounters[FT_MAX_COUNTERS]; /* Fixes */
	/** motor cmdid structure to control op execution */
	struct master_motor {
		uint32_t mcmdid; /*!< Motor cmdid for comparision */
		uint32_t lcmdid; /*!< Last executed client motor cmdid */
		uint32_t ncmdid; /*!< New stared client motor cmdid */
	} mmotor_cmdid[FT_MAX_MOTORS];
	uint32_t ncmdid; /* Common local cmdid */
	uint8_t  mmask; /* Motor groups */

	struct ft_motor_break_info break_info[FT_MAX_MOTORS];
	struct ft_shm_stat stat;
	uint32_t task_times[256];

	/* Slaves */
	struct ft_shm_txt_remote slaves[FT_MAX_SLAVES];
} txt2_server;

#endif /* FTINC_FTSERVERMEM_H_ */
