/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @defgroup Common_FT_API_Implementation_Includes
 * @{
 */

/**
 * @file fttransport.h
 * @brief A7-M4 transport
 */
#ifndef _FTOP_TRANSPORT
#define _FTOP_TRANSPORT

#include <stdint.h>

/**
 * @addtogroup Local_Communication
 * @{
 */

/* typedef */

/** A7-M4 transport structure */
typedef struct ftop_transport {
    int fid;	/*!< File id */
    int lfid;	/*!< Lock file id */
    int cid;    /*!< Channel id */
} ftop_transport;

typedef struct ftop_client ftop_client; /*!< Forward declaration */

/* Exported functions */

/**
 * @brief Get transport
 * @param host unused
 * @param port unused
 * @return transport structure
 */
ftop_transport* ftop_socket(const char *host, int port);

/**
 * @brief Send packet to M4
 * @param transport transport structure
 * @param packet packet with command
 * @param size command size
 * @return packet length
 */
int ftop_socket_send(void *transport, void *packet, int size);

/**
 * @brief Send RPMSG response to A7 client, wakeup sendThread if required
 * @param ftc client
 * @param buffer buffer with response
 * @param len length of information
 * @return length of information
 */
int ftserver_send_response(ftop_client *ftc, void *buffer, int len);

/**
 * @brief Receive packet from M4
 * @param transport transport structure
 * @param packet buffer for packet
 * @param len buffer length
 * @return packet length
 */
int ftop_socket_receive(void *transport, void *packet, int len);

/**
 * @brief Free communication channel
 * @return success 0
 */
int ftop_socket_free(void *transport);

/**
 * @brief Try to lock client
 * @param lid lock file
 * @param client client id
 * @return success or fail
 */
int ftop_lock_client(int lid, int client);

/**
 * @brief Try to unlock client
 * @param lid lock file
 * @param client client id
 * @return success or fail
 */
int ftop_unlock_client(int lid, int client);

/**
 * @brief Try to lock client
 * @param client client id
 * @return file id
 */
int ftop_open_client_file(int client);

/**
 * @brief Rpmsg command server initialization procedure
 */
void Init_Rpmsg_Server(void);

#endif /* _FTOP_TRANSPORT */

/** @} */

/** @} */
