#include "ftcmdline.h"

#if CMDLINE_MAXBUFFER > 0
static uint8_t au8InternBuffer[CMDLINE_MAXBUFFER];
static volatile uint16_t u16CurrentPos;
#endif

static stc_ftctrl_cmdline_item_t* pRoot = NULL;

static void FtCmdLine_Help(char* args[], uint8_t argv);


#define COMMAND_COUNT 1
static stc_ftctrl_cmdline_item_t astcCommands[] = {{FtCmdLine_Help,"help","Print help"}};
static void putString(char* pcString);


static void FtCmdLine_Help(char* args[], uint8_t argv)
{
    stc_ftctrl_cmdline_item_t* pCurrent = pRoot;
    uint8_t i;
    putString("\r\n");
    putString("\r\n");
    putString("Available command line options:\r\n");
    while(pCurrent != NULL)
    {
            if ((NULL == pCurrent->pcCommand) || (NULL == pCurrent->pfnCallback))
            {
                putString("\r\n");
                putString(pCurrent->pcHelpText);
                putString("\r\n");
            }
            else
            {
                putString("  ");
                putString(pCurrent->pcCommand);
                for(i = strlen(pCurrent->pcCommand);i < 20;i++)
                {
                    putString(" ");
                }
                putString(pCurrent->pcHelpText);
                putString("\r\n");
            }
        pCurrent = pCurrent->nextListItm;
    }
    putString("\r\n");
    putString("\r\n");
}

static void putString(char* pcString)
{
    printf("%s",pcString);
}

void FtCmdLine_Init(void)
{
    memset(au8InternBuffer,0,CMDLINE_MAXBUFFER);
    FtCmdLine_AddCommands(astcCommands,COMMAND_COUNT);
}

void FtCmdLine_AddCommands(stc_ftctrl_cmdline_item_t apfnCommands[], uint8_t u8CommandCount)
{
    uint8_t i;
    stc_ftctrl_cmdline_item_t* pLastItem = pRoot;
    if (pRoot != NULL)
    {
        while(pLastItem->nextListItm != NULL)
        {
            pLastItem = pLastItem->nextListItm;
        }
    }
    for(i = 0;i <u8CommandCount;i++)
    {
        if (pRoot == NULL)
        {
            pRoot = &apfnCommands[i];
            pLastItem = pRoot;
            apfnCommands[i].nextListItm = NULL;
        }
        else
        {
            pLastItem->nextListItm = &apfnCommands[i];
            pLastItem = pLastItem->nextListItm;
        }
    }
}

void FtCmdLine_Execute(char *pcString)
{
    char* pcTmp = pcString;
    while(*pcTmp != '\0')
    {
        if (*pcTmp == ' ')
        {
            *pcTmp = '\0';
            pcTmp++;
            FtCmdLine_ExecuteCommand(pcString,pcTmp);
            return;
        }
        pcTmp++;
    }
    FtCmdLine_ExecuteCommand(pcString,NULL);
}

void FtCmdLine_ExecuteCommand(char* pcCommand,char* pcArgumentString)
{
    char* args[CMDLINE_MAXCMDARGS];
    volatile uint8_t argv = 0;
    char* pcTmp = pcArgumentString;
    if (pcArgumentString != NULL)
    {
      while(*pcTmp != '\0')
      {
        if (*pcTmp != ' ')
        {
            args[argv] = pcTmp;
            argv++;
            while(*pcTmp != '\0')
            {
                if (*pcTmp == ' ')
                {
                    *pcTmp = '\0';
                    pcTmp++;
                    break;
                }
                pcTmp++;
            }
        }
        else
        {
            pcTmp++;
        }
      }
    }
    FtCmdLine_ExecuteCommandArgs(pcCommand,args,argv);
}

void FtCmdLine_ExecuteCommandArgs(char* pcCommand,char* args[],uint8_t argv)
{
    uint8_t i;
    stc_ftctrl_cmdline_item_t* pCurrent = pRoot;
    while(pCurrent != NULL)
    {
        if (strcmp(pcCommand,pCurrent->pcCommand) == 0)
        {
            if (pCurrent->pfnCallback != NULL)
            {
                pCurrent->pfnCallback(args,argv);
                return;
            }
        }
        pCurrent = pCurrent->nextListItm;
    }
    putString("Command not found: ");
    putString(pcCommand);
    putString("\r\n");

}

void FtCmdLine_Update(void)
{
    static uint16_t u16LastCheck = 0;
    if (u16LastCheck != u16CurrentPos)
    {
        for(;u16LastCheck <= u16CurrentPos;u16LastCheck++)
        {
            if ((au8InternBuffer[u16LastCheck] == '\r') || (au8InternBuffer[u16LastCheck] == '\n') ||(u16LastCheck == CMDLINE_MAXBUFFER))
            {
                au8InternBuffer[u16LastCheck] = '\0';
                FtCmdLine_Execute((char*)&au8InternBuffer[0]);
                memcpy(&au8InternBuffer[0],&au8InternBuffer[u16LastCheck],(CMDLINE_MAXBUFFER - u16LastCheck));
                u16LastCheck = 0;
                u16CurrentPos = 0;
            }
            if (au8InternBuffer[u16LastCheck] == 0) break;
        }
    }
}

#if CMDLINE_MAXBUFFER > 0
void FtCmdLine_AddChar(char cChar)
{
    if ((cChar == '\b') && (u16CurrentPos > 0))
    {
        u16CurrentPos--;
    }
    
    if (u16CurrentPos < (CMDLINE_MAXBUFFER - 1))
    {
        au8InternBuffer[u16CurrentPos] = cChar;
        au8InternBuffer[u16CurrentPos+1] = '\0';
        u16CurrentPos++;
    }
}
void FtCmdLine_AddString(char* pcChar)
{
    while(*pcChar != '\0')
    {
        FtCmdLine_AddChar(*pcChar);
        pcChar++;
    }
}
#endif