/*
 * Copyright (c) 2023 Accso. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file ftutil/ftctrl.cpp
 * @brief FT Control utility
 */


#include "ftcmdline.h"
#include <iostream>
#include <string>
#include <stdio.h>
#include <unistd.h>

#ifdef __cplusplus
    extern "C" {
        #include "ftlock.h"
    }
    #include "ft.hpp"
    #include "ftop.h"
#else
    #include "ftlock.h"
    #include "ft.h"
#endif

#ifdef __cplusplus
    using namespace ft;
#endif


#define SERVO_MIN 300UL
#define SERVO_MAX 2700UL

#ifdef __cplusplus
TXT txt2;
#else
void *txt2;
#endif

static void cmdServoUs(char* args[], uint8_t argc);
static void cmdServo9Bit(char* args[], uint8_t argc);

#define COMMANDS_SERVO_COUNT (sizeof(astcCommandsServo) / sizeof(astcCommandsServo[0]))
static stc_ftctrl_cmdline_item_t astcCommandsServo[] = {
                                                        {cmdServoUs,"servo_us","<num> <position in us>"},
                                                        {cmdServo9Bit,"servo_9bit","<num> <position 0-512"}
                                                       };


static void cmdServoUs(char* args[], uint8_t argc)
{
    int servoNum = 1;
    int servoPos=1485;
    if (argc < 2)
    {
        printf("ERROR\n");
        return;
    }

    servoNum = atoi(args[1]);
    servoPos = atoi(args[2]);

    if ((servoNum < 1) || (servoNum > 3))
    {
        printf("ERROR\n");
        return;
    }

    if ((servoPos < SERVO_MIN) || (servoPos > SERVO_MAX))
    {
        printf("ERROR\n");
        return;
    }

    if (ftlock_open() < 0)
    {
        printf("ERROR\n");
        return;
    }
    
    Servo s { txt2, servoNum };

    txt2.update_config();
    s.setPwmUs(servoPos);

    printf("OK\n");
    return;
}

static void cmdServo9Bit(char* args[], uint8_t argc)
{
int servoNum = 1;
    int servoPos=1485;
    if (argc < 2)
    {
        printf("ERROR\n");
        return;
    }

    servoNum = atoi(args[1]);
    servoPos = atoi(args[2]);

    if ((servoNum < 1) || (servoNum > 3))
    {
        printf("ERROR\n");
        return;
    }

    if ((servoPos < SERVO_MIN) || (servoPos > SERVO_MAX))
    {
        printf("ERROR\n");
        return;
    }

    if (ftlock_open() < 0)
    {
        printf("ERROR\n");
        return;
    }
    Servo s { txt2, servoNum };

    txt2.update_config();
    s.setPwm(servoPos);

    ftlock_unlock(); // can be skipped in this case
    ftlock_close();  // can be skipped in this case
    printf("OK\n");
    return;
}

/**
 * @brief Standard main procedure
 * @param [in] argc number of arguments
 * @param [in] argv arguments
 * @return Error code
 */
int main(int argc, char **argv)
{
    FtCmdLine_Init();
    FtCmdLine_AddCommands(astcCommandsServo,COMMANDS_SERVO_COUNT);
    #ifndef __cplusplus
    txt2 = ftop_alloc_txt("auto", 65000);
    if (ltxt == NULL) {
        fprintf(stderr,"Can't conect %s\n", argv[1]);
        return 1;
    }
    #endif

    if (argc > 2)
    {
        FtCmdLine_ExecuteCommandArgs(argv[1],&argv[1],argc-1);
    } else if (argc > 1)
    {
        FtCmdLine_ExecuteCommandArgs(argv[1],&argv[1],0);
    } else
    {
        FtCmdLine_ExecuteCommandArgs("help",NULL,0);
    }

    ftlock_unlock(); // can be skipped in this case
    ftlock_close();  // can be skipped in this case
}