/*
 * Copyright (c) 2020 RTSoft. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @file mcommon.h
 * @brief Common defines per rev0/rev1
 */

#ifndef INC_MCOMMON_H_
#define INC_MCOMMON_H_

/* Defines */

#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define CHECK_PIN2(n) (((n) & 0x1) ? 0 : 1)
#define CHECK_PIN4(n) (((n) & 0x3) ? CHECK_PIN2(n) : (CHECK_PIN2((n) >> 2) + 2))
#define CHECK_PIN8(n) (((n) & 0xf) ? CHECK_PIN4(n) : (CHECK_PIN4((n) >> 4) + 4))
#define CHECK_PIN(n) (((n) & 0xff) ? CHECK_PIN8(n) : (CHECK_PIN8((n) >> 8) + 8))
#define REMUINT(y)
#define OPEN_PIN(y) REMUINT y
#endif // DOXYGEN_SHOULD_SKIP_THIS

#define NPIN(x) CHECK_PIN(OPEN_PIN x) /*!< Convert cubemx pin bit to bit number */

#define I1I8TimeResolution   200 /*!< I1-I8 clibration time resolution in ns */
#define I1I8Delay    2500		 /*!< Initial value for I1-I8 period ns */
#define SERVO_PERIOD 20000		 /*!< total servo period 20 ms */
#define SERVO_MIN_PULSE 610		 /*!< minimal servo pulse */
#define SERVO_MAX_PULSE 2360	 /*!< maximum servo pulse */
#define SERVO_MID_PULSE ((SERVO_MIN_PULSE + SERVO_MAX_PULSE)/2)	/*!< medium servo pulse */

#if TXT2_BOARD_REVISION
#define COLLECT_TASK_INFO 1	/*!< Collect task time info for board */
#else
#define COLLECT_TASK_INFO 0 /*!< Collect task time info for board is unavailable for rev0 */
#endif /* TXT2_BOARD_REVISION */

uint32_t buid; /* Board uid */

#endif /* INC_MCOMMON_H_ */
